#!/usr/bin/env python3

from numpy import uint64

# taken from:
# https://stackoverflow.com/questions/47678568/deterministic-pseudorandom-number-generation/47681859#47681859


def hash_2d(i, j):
    x = i + (j << 32)
    x = (x ^ (x >> 30)) * (0xbf58476d1ce4e5b9)
    x = (x ^ (x >> 27)) * (0x94d049bb133111eb)
    x = x ^ (x >> 31)
    return uint64(x % (1 << 64))


def hash_3d(i, j, k):
    x = i + (j << 32) + (k << 64)  # hint: maybe produces a lot of costs
    x = (x ^ (x >> 30)) * (0xbf58476d1ce4e5b9)
    x = (x ^ (x >> 27)) * (0x94d049bb133111eb)
    x = x ^ (x >> 31)
    return uint64(x % (1 << 64))


def hash_nd(v):
    d = len(v)
    if d == 2:
        return hash_2d(int(v[0]), int(v[1]))
    if d == 3:
        return hash_3d(int(v[0]), int(v[1]), int(v[2]))
    x = v[0] + sum([v[i] << (16 << i) for i in range(1, d)])
    x = (x ^ (x >> 30)) * (0xbf58476d1ce4e5b9)
    x = (x ^ (x >> 27)) * (0x94d049bb133111eb)
    x = x ^ (x >> 31)
    return uint64(x % (1 << 64))
